@@ -68,7 +68,7 @@
 #include "raptoptions.h"
 
 
-static char descrBuffer[4096];
+static char descrBuffer[8192];
 
 static char *parseDescription(string descr);
 
@@ -398,7 +398,7 @@ vector<RPackage::DepInformation> RPackage::enumRDeps()
 	 dep.versionComp = "";
       }
 
-      dep.typeStr=_("Depended on by");
+      dep.typeStr=_("Dependency of");
       dep.name = D.ParentPkg().Name();
 
       if(Trg->VersionList == 0)
