@@ -38,6 +38,15 @@
 
 enum { FONT_DEFAULT, FONT_TERMINAL };
 
+const char * RGPreferencesWindow::column_names[] = 
+   {"status", "name", "instVer", "availVer", "instSize", "descr", NULL };
+
+const char *RGPreferencesWindow::column_visible_names[] = 
+   {_("Status"), _("Package Name"), _("Installed Version"), 
+    _("Available Version"), _("Installed Size"), _("Description"), NULL };
+
+
+
 void RGPreferencesWindow::onArchiveSelection(GtkWidget *self, void *data)
 {
    RGPreferencesWindow *me =
@@ -45,14 +54,10 @@ void RGPreferencesWindow::onArchiveSelection(GtkWidget *self, void *data)
    //cout << "void RGPreferencesWindow::onArchiveSelection()" << endl;
    //cout << "data is: " << (char*)data << endl;
 
-   string s = (char *)data;
-   if (s.empty()) {
-      _config->Clear("APT::Default-Release");
-      _config->Clear("Synaptic::DefaultDistro");
-   } else {
-      _config->Set("APT::Default-Release", s);
-      _config->Set("Synaptic::DefaultDistro", s);
-   }
+   if(me->_blockAction)
+      return;
+
+   me->_defaultDistro = (char *)data;
    me->distroChanged = true;
 }
 
@@ -63,6 +68,17 @@ void RGPreferencesWindow::applyProxySettings()
    gchar *s;
    int httpPort, ftpPort;
 
+   // save apts settings here
+   static bool firstRun = true;
+   static string aptHttpProxy, aptFtpProxy;
+
+   if(firstRun) {
+      aptHttpProxy = _config->Find("Acquire::http::Proxy");
+      aptFtpProxy = _config->Find("Acquire::ftp::Proxy");
+      
+      firstRun = false;
+   }
+
    bool useProxy = _config->FindB("Synaptic::useProxy", false);
    // now set the stuff for apt
    if (useProxy) {
@@ -95,95 +111,78 @@ void RGPreferencesWindow::applyProxySettings()
       }
       g_strfreev(noProxyArray);
    } else {
-      _config->Clear("Acquire::http::Proxy");
-      _config->Clear("Acquire::ftp::Proxy");
+      //FIXME: we can't just clean here as apt may have it's own proxy 
+      // settings!
+      _config->Set("Acquire::http::Proxy",aptHttpProxy);
+      _config->Set("Acquire::ftp::Proxy", aptFtpProxy);
    }
 }
 
-void RGPreferencesWindow::saveAction(GtkWidget *self, void *data)
+void RGPreferencesWindow::saveGeneral()
 {
-   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
    bool newval;
    int i;
 
-   // cache
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_cacheClean));
-   _config->Set("Synaptic::CleanCache", newval ? "true" : "false");
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_cacheAutoClean));
-   _config->Set("Synaptic::AutoCleanCache", newval ? "true" : "false");
+   // show package properties in main window
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionShowAllPkgInfoInMain));
+   _config->Set("Synaptic::ShowAllPkgInfoInMain", newval ? "true" : "false");
+   // apply the changes
+   GtkWidget *notebook = glade_xml_get_widget(_mainWin->getGladeXML(),
+					      "notebook_pkginfo");
+   gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), newval);
+   GtkWidget *box = glade_xml_get_widget(_mainWin->getGladeXML(),
+					 "vbox_pkgdescr");
+   if(newval) {
+      gtk_container_set_border_width(GTK_CONTAINER(box), 6);
+   } else {
+      gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
+      gtk_container_set_border_width(GTK_CONTAINER(box), 0);
+   }
 
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionUseRegexp));
-   _config->Set("Synaptic::UseRegexp", newval ? "true" : "false");
-
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionAskRelated));
+   // Ask to confirm changes also affecting other packages
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionAskRelated));
    _config->Set("Synaptic::AskRelated", newval ? "true" : "false");
 
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionUseTerminal));
-   _config->Set("Synaptic::UseTerminal", newval ? "true" : "false");
-
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionCheckRecom));
+   // Consider recommended packages as dependencies
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionCheckRecom));
    _config->Set("Synaptic::UseRecommends", newval ? "true" : "false");
 
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionAskQuit));
-   _config->Set("Synaptic::AskQuitOnProceed", newval ? "true" : "false");
-
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionOneClick));
+   // Clicking on the status icon marks the most likely action
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionOneClick));
    _config->Set("Synaptic::OneClickOnStatusActions", newval ? "true" : "false");
 
-
-   int maxUndo = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(me->_maxUndoE));
-   _config->Set("Synaptic::undoStackSize", maxUndo);
-
-   int delAction = gtk_option_menu_get_history(GTK_OPTION_MENU(me->_optionmenuDel));
+   // Removal of packages: 
+   int delAction= gtk_option_menu_get_history(GTK_OPTION_MENU(_optionmenuDel));
    // ugly :( but we need this +2 because RGPkgAction starts with 
    //         "keep","install"
    delAction += 2;
    _config->Set("Synaptic::delAction", delAction);
 
-   // save the colors
-   RGPackageStatus::pkgStatus.saveColors();
-   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_optionUseStatusColors));
-   _config->Set("Synaptic::UseStatusColors", newval ? "true" : "false");
-
+   // System upgrade:
    // upgrade type, (ask=-1,normal=0,dist-upgrade=1)
-   i = gtk_option_menu_get_history(
-      GTK_OPTION_MENU(glade_xml_get_widget(me->_gladeXML,
-                                           "optionmenu_upgrade_method")));
+   i = gtk_option_menu_get_history(GTK_OPTION_MENU(glade_xml_get_widget(_gladeXML, "optionmenu_upgrade_method")));
    _config->Set("Synaptic::upgradeType", i - 1);
 
-   // proxy stuff
-   bool useProxy;
-   const gchar *http, *ftp, *noProxy;
-   int httpPort, ftpPort;
+   // Number of undo operations:
+   int maxUndo = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(_maxUndoE));
+   _config->Set("Synaptic::undoStackSize", maxUndo);
 
-   useProxy = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(me->_useProxy));
-   _config->Set("Synaptic::useProxy", useProxy);
-   // http
-   http = gtk_entry_get_text(
-      GTK_ENTRY(glade_xml_get_widget(me->_gladeXML, "entry_http_proxy")));
-   _config->Set("Synaptic::httpProxy", http);
-   httpPort = (int) gtk_spin_button_get_value(
-      GTK_SPIN_BUTTON(glade_xml_get_widget(me->_gladeXML,
-                                           "spinbutton_http_port")));
-   _config->Set("Synaptic::httpProxyPort", httpPort);
-   // ftp
-   ftp = gtk_entry_get_text(
-      GTK_ENTRY(glade_xml_get_widget(me->_gladeXML, "entry_ftp_proxy")));
-   _config->Set("Synaptic::ftpProxy", ftp);
-   ftpPort = (int) gtk_spin_button_get_value(
-      GTK_SPIN_BUTTON(glade_xml_get_widget(me->_gladeXML,
-                                           "spinbutton_ftp_port")));
-   _config->Set("Synaptic::ftpProxyPort", ftpPort);
-   noProxy = gtk_entry_get_text(
-      GTK_ENTRY(glade_xml_get_widget(me->_gladeXML, "entry_no_proxy")));
-   _config->Set("Synaptic::noProxy", noProxy);
+   // Apply changes in a terminal window
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionUseTerminal));
+   _config->Set("Synaptic::UseTerminal", newval ? "true" : "false");
 
-   me->applyProxySettings();
+   // Ask to quit after the changes have been applied successfully
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionAskQuit));
+   _config->Set("Synaptic::AskQuitOnProceed", newval ? "true" : "false");
 
-   // font stuff
-   newval = gtk_toggle_button_get_active(
-      GTK_TOGGLE_BUTTON(glade_xml_get_widget(me->_gladeXML,
-                                             "checkbutton_user_font")));
+}
+
+void RGPreferencesWindow::saveColumnsAndFonts() 
+{
+   bool newval;
+
+   // Use custom application font
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(_gladeXML, "checkbutton_user_font")));
    _config->Set("Synaptic::useUserFont", newval);
 
    GValue value = { 0, };
@@ -201,26 +200,122 @@ void RGPreferencesWindow::saveAction(GtkWidget *self, void *data)
    }
    g_value_unset(&value);
 
-   newval = gtk_toggle_button_get_active(
-      GTK_TOGGLE_BUTTON(glade_xml_get_widget(me->_gladeXML,
-                                             "checkbutton_user_terminal_font")));
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(_gladeXML, "checkbutton_user_terminal_font")));
    _config->Set("Synaptic::useUserTerminalFont", newval);
+   
 
+   // treeviewstuff 
+   // get from GtkListStore
+   GtkListStore *store = _listColumns;
+   GtkTreeIter iter;
+   int i=0;
+   char *column_name, *config_name;
+   bool visible;
+   gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+   do {
+      gtk_tree_model_get (GTK_TREE_MODEL(store), &iter,
+			  TREE_CHECKBOX_COLUMN, &visible,
+			  TREE_NAME_COLUMN, &column_name,
+			  -1);
+
+      // pos
+      config_name = g_strdup_printf("Synaptic::%sColumnPos",column_name);
+      _config->Set(config_name, i);
+      //cout << column_name << " : " << i << endl;
+      g_free(config_name);
+      
+      // visible
+      config_name = g_strdup_printf("Synaptic::%sColumnVisible",column_name);
+      _config->Set(config_name, visible);
+      g_free(config_name);
+
+      i++;
+   } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));
 
    // rebuild the treeview
-   me->saveTreeViewValues();
-   me->_mainWin->rebuildTreeView();
+   _mainWin->rebuildTreeView();
+}
 
-   switch (me->_synapticLayout) {
-      case LAYOUT_VPANED:
-         _config->Set("Synaptic::MainName", "main_vpaned");
-         break;
-      case LAYOUT_HPANED:
-         _config->Set("Synaptic::MainName", "main_hpaned");
-         break;
-      default:
-         cerr << "uuuoooohhhh" << endl;
+void RGPreferencesWindow::saveColors()
+{
+   bool newval;
+
+   // save the colors
+   RGPackageStatus::pkgStatus.saveColors();
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_optionUseStatusColors));
+   _config->Set("Synaptic::UseStatusColors", newval ? "true" : "false");
+}
+
+void RGPreferencesWindow::saveTempFiles()
+{
+   bool newval;
+
+   // cache
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_cacheClean));
+   _config->Set("Synaptic::CleanCache", newval ? "true" : "false");
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_cacheAutoClean));
+   _config->Set("Synaptic::AutoCleanCache", newval ? "true" : "false");
+   
+}
+
+void RGPreferencesWindow::saveNetwork()
+{
+   // proxy stuff
+   bool useProxy;
+   const gchar *http, *ftp, *noProxy;
+   int httpPort, ftpPort;
+
+   useProxy = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_useProxy));
+   _config->Set("Synaptic::useProxy", useProxy);
+   // http
+   http = gtk_entry_get_text(
+      GTK_ENTRY(glade_xml_get_widget(_gladeXML, "entry_http_proxy")));
+   _config->Set("Synaptic::httpProxy", http);
+   httpPort = (int) gtk_spin_button_get_value(
+      GTK_SPIN_BUTTON(glade_xml_get_widget(_gladeXML,
+                                           "spinbutton_http_port")));
+   _config->Set("Synaptic::httpProxyPort", httpPort);
+   // ftp
+   ftp = gtk_entry_get_text(
+      GTK_ENTRY(glade_xml_get_widget(_gladeXML, "entry_ftp_proxy")));
+   _config->Set("Synaptic::ftpProxy", ftp);
+   ftpPort = (int) gtk_spin_button_get_value(
+      GTK_SPIN_BUTTON(glade_xml_get_widget(_gladeXML,
+                                           "spinbutton_ftp_port")));
+   _config->Set("Synaptic::ftpProxyPort", ftpPort);
+   noProxy = gtk_entry_get_text(
+      GTK_ENTRY(glade_xml_get_widget(_gladeXML, "entry_no_proxy")));
+   _config->Set("Synaptic::noProxy", noProxy);
+
+   applyProxySettings();
+
+}
+ 
+void RGPreferencesWindow::saveExpert()
+{
+   int nr;
+   nr = gtk_option_menu_get_history(GTK_OPTION_MENU(_optionmenuDefaultDistro));
+   
+   if (_defaultDistro.empty()) {
+      _config->Clear("APT::Default-Release");
+      _config->Clear("Synaptic::DefaultDistro");
+   } else {
+      _config->Set("APT::Default-Release", _defaultDistro);
+      _config->Set("Synaptic::DefaultDistro", _defaultDistro);
    }
+}
+
+
+void RGPreferencesWindow::saveAction(GtkWidget *self, void *data)
+{
+   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
+
+   me->saveGeneral();
+   me->saveColumnsAndFonts();
+   me->saveColors();
+   me->saveTempFiles();
+   me->saveNetwork();
+   me->saveExpert();
 
    if (!RWriteConfigFile(*_config)) {
       _error->Error(_("An error occurred while saving configurations."));
@@ -297,31 +392,42 @@ void RGPreferencesWindow::clearCacheAction(GtkWidget *self, void *data)
    me->_lister->cleanPackageCache(true);
 }
 
-
-void RGPreferencesWindow::show()
+void RGPreferencesWindow::readGeneral()
 {
-   bool postClean, postAutoClean;
-   string str;
-   int i;
-   bool b;
-   distroChanged = false;
+   // Allow regular expressions in searches and filters
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionShowAllPkgInfoInMain),
+                                _config->FindB("Synaptic::ShowAllPkgInfoInMain", false));
 
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheClean),
-                                _config->FindB("Synaptic::CleanCache", false));
-
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheAutoClean),
-                                _config->FindB("Synaptic::AutoCleanCache",
-                                               false));
-
-
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionUseRegexp),
-                                _config->FindB("Synaptic::UseRegexp", false));
-
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionUseStatusColors),
-                                _config->FindB("Synaptic::UseStatusColors",
-                                               true));
+   // Ask to confirm changes also affecting other packages
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionAskRelated),
                                 _config->FindB("Synaptic::AskRelated", true));
+
+   // Consider recommended packages as dependencies
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionCheckRecom),
+                                _config->FindB("Synaptic::UseRecommends",
+                                               false));
+
+   // Clicking on the status icon marks the most likely action
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionOneClick),
+                                _config->FindB("Synaptic::OneClickOnStatusActions",
+                                               false));
+
+   // Removal of packages: 
+   int delAction = _config->FindI("Synaptic::delAction", PKG_DELETE);
+   // now set the optionmenu
+   // ugly :( but we need this -2 because RGPkgAction starts with 
+   //         "keep","install"
+   gtk_option_menu_set_history(GTK_OPTION_MENU(_optionmenuDel), delAction - 2);
+
+
+   // System upgrade:
+   // upgradeType (ask=-1,normal=0,dist-upgrade=1)
+   int i = _config->FindI("Synaptic::upgradeType", -1);
+   gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(_gladeXML, "optionmenu_upgrade_method")), i + 1);
+
+
+   // Number of undo operations:
+
 #ifdef HAVE_RPM
    int UndoStackSize = 3;
 #else
@@ -331,6 +437,8 @@ void RGPreferencesWindow::show()
                              _config->FindI("Synaptic::undoStackSize",
                                             UndoStackSize));
 
+   
+   // Apply changes in a terminal window
    bool UseTerminal = false;
 #ifndef HAVE_TERMINAL
    gtk_widget_set_sensitive(GTK_WIDGET(_optionUseTerminal), false);
@@ -343,76 +451,17 @@ void RGPreferencesWindow::show()
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionUseTerminal),
                                 _config->FindB("Synaptic::UseTerminal",
                                                UseTerminal));
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionCheckRecom),
-                                _config->FindB("Synaptic::UseRecommends",
-                                               false));
 
+   // Ask to quit after the changes have been applied successfully
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionAskQuit),
                                 _config->FindB("Synaptic::AskQuitOnProceed",
                                                false));
+}
 
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionOneClick),
-                                _config->FindB("Synaptic::OneClickOnStatusActions",
-                                               false));
-
-
-   postClean = _config->FindB("Synaptic::CleanCache", false);
-   postAutoClean = _config->FindB("Synaptic::AutoCleanCache", false);
-
-   if (postClean)
-      gtk_button_clicked(GTK_BUTTON(_cacheClean));
-   else if (postAutoClean)
-      gtk_button_clicked(GTK_BUTTON(_cacheAutoClean));
-   else
-      gtk_button_clicked(GTK_BUTTON(_cacheLeave));
-
-   // upgradeType (ask=-1,normal=0,dist-upgrade=1)
-   i = _config->FindI("Synaptic::upgradeType", -1);
-   gtk_option_menu_set_history(
-      GTK_OPTION_MENU(glade_xml_get_widget(_gladeXML,
-                                           "optionmenu_upgrade_method")), i + 1);
-
-
-   // proxy stuff
-   bool useProxy = _config->FindB("Synaptic::useProxy", false);
-   gtk_toggle_button_set_active(
-      GTK_TOGGLE_BUTTON(glade_xml_get_widget(_gladeXML, "radio_use_proxy")),
-      useProxy);
-   gtk_widget_set_sensitive(glade_xml_get_widget(_gladeXML, "table_proxy"),
-                            useProxy);
-   str = _config->Find("Synaptic::httpProxy", "");
-   gtk_entry_set_text(GTK_ENTRY
-                      (glade_xml_get_widget(_gladeXML, "entry_http_proxy")),
-                      str.c_str());
-   i = _config->FindI("Synaptic::httpProxyPort", 3128);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON
-                             (glade_xml_get_widget
-                              (_gladeXML, "spinbutton_http_port")), i);
-
-   str = _config->Find("Synaptic::ftpProxy", "");
-   gtk_entry_set_text(
-      GTK_ENTRY(glade_xml_get_widget(_gladeXML, "entry_ftp_proxy")), str.c_str());
-   i = _config->FindI("Synaptic::ftpProxyPort", 3128);
-   gtk_spin_button_set_value(
-      GTK_SPIN_BUTTON(glade_xml_get_widget(_gladeXML, "spinbutton_ftp_port")), i);
-   str = _config->Find("Synaptic::noProxy", "");
-   gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(_gladeXML,
-                                                     "entry_no_proxy")),
-                      str.c_str());
-   // main layout
-   string mainName = _config->Find("Synaptic::MainName", "main_hpaned");
-   if (mainName == "main_vpaned") {
-      gtk_button_clicked(GTK_BUTTON(glade_xml_get_widget(_gladeXML,
-                                                         "button_vpaned")));
-      _synapticLayout = LAYOUT_VPANED;
-   } else {
-      gtk_button_clicked(GTK_BUTTON
-                         (glade_xml_get_widget(_gladeXML, "button_hpaned")));
-      _synapticLayout = LAYOUT_HPANED;
-   }
-
+void RGPreferencesWindow::readColumnsAndFonts()
+{
    // font stuff
-   b = _config->FindB("Synaptic::useUserFont", false);
+   bool b = _config->FindB("Synaptic::useUserFont", false);
    gtk_toggle_button_set_active(
       GTK_TOGGLE_BUTTON(glade_xml_get_widget(_gladeXML,
                                              "checkbutton_user_font")), b);
@@ -421,91 +470,20 @@ void RGPreferencesWindow::show()
       GTK_TOGGLE_BUTTON(glade_xml_get_widget(_gladeXML,
                                              "checkbutton_user_terminal_font")), b);
 
-   RGWindow::show();
 }
 
-void RGPreferencesWindow::readTreeViewValues()
-{
-   GtkWidget *b;
-   int pos;
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_status");
-   assert(b);
-   pos = _config->FindI("Synaptic::statusColumnPos", 0);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON(b), pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_name");
-   assert(b);
-   pos = _config->FindI("Synaptic::nameColumnPos", 1);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON(b), pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_instver");
-   assert(b);
-   pos = _config->FindI("Synaptic::instVerColumnPos", 2);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON(b), pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_availver");
-   assert(b);
-   pos = _config->FindI("Synaptic::availVerColumnPos", 3);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON(b), pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_instsize");
-   assert(b);
-   pos = _config->FindI("Synaptic::instSizeColumnPos", 4);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON(b), pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_descr");
-   assert(b);
-   pos = _config->FindI("Synaptic::descrColumnPos", 5);
-   gtk_spin_button_set_value(GTK_SPIN_BUTTON(b), pos);
-
-}
-
-void RGPreferencesWindow::saveTreeViewValues()
-{
-   GtkWidget *b;
-   int pos;
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_status");
-   assert(b);
-   pos = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(b));
-   _config->Set("Synaptic::statusColumnPos", pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_name");
-   assert(b);
-   pos = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(b));
-   _config->Set("Synaptic::nameColumnPos", pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_instver");
-   assert(b);
-   pos = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(b));
-   _config->Set("Synaptic::instVerColumnPos", pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_availver");
-   assert(b);
-   pos = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(b));
-   _config->Set("Synaptic::availVerColumnPos", pos);
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_instsize");
-   assert(b);
-   pos = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(b));
-   _config->Set("Synaptic::instSizeColumnPos", pos);
-
-
-   b = glade_xml_get_widget(_gladeXML, "spinbutton_descr");
-   assert(b);
-   pos = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(b));
-   _config->Set("Synaptic::descrColumnPos", pos);
-}
-
-
-
 void RGPreferencesWindow::readColors()
 {
    GdkColor *color;
    gchar *color_button;
    GtkWidget *button = NULL;
 
+   // Color packages by their status
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionUseStatusColors),
+                                _config->FindB("Synaptic::UseStatusColors",
+                                               true));
+
+   // color buttons
    for (int i = 0; i < RGPackageStatus::N_STATUS_COUNT; i++) {
       color_button =
          g_strdup_printf("button_%s_color",
@@ -520,8 +498,254 @@ void RGPreferencesWindow::readColors()
       }
       g_free(color_button);
    }
+
 }
 
+void RGPreferencesWindow::readTempFiles()
+{
+   // <b>Temporary Files</b>
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheClean),
+                                _config->FindB("Synaptic::CleanCache", false));
+
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheAutoClean),
+                                _config->FindB("Synaptic::AutoCleanCache",
+                                               false));
+
+   bool postClean = _config->FindB("Synaptic::CleanCache", false);
+   bool postAutoClean = _config->FindB("Synaptic::AutoCleanCache", false);
+
+   if (postClean)
+      gtk_button_clicked(GTK_BUTTON(_cacheClean));
+   else if (postAutoClean)
+      gtk_button_clicked(GTK_BUTTON(_cacheAutoClean));
+   else
+      gtk_button_clicked(GTK_BUTTON(_cacheLeave));
+
+}
+
+void RGPreferencesWindow::readNetwork()
+{
+   // proxy stuff
+   bool useProxy = _config->FindB("Synaptic::useProxy", false);
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(_gladeXML, "radio_use_proxy")), useProxy);
+   gtk_widget_set_sensitive(glade_xml_get_widget(_gladeXML, "table_proxy"),
+                            useProxy);
+   string str = _config->Find("Synaptic::httpProxy", "");
+   gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(_gladeXML, "entry_http_proxy")), str.c_str());
+   int i = _config->FindI("Synaptic::httpProxyPort", 3128);
+   gtk_spin_button_set_value(GTK_SPIN_BUTTON
+                             (glade_xml_get_widget
+                              (_gladeXML, "spinbutton_http_port")), i);
+
+   str = _config->Find("Synaptic::ftpProxy", "");
+   gtk_entry_set_text(
+      GTK_ENTRY(glade_xml_get_widget(_gladeXML, "entry_ftp_proxy")), str.c_str());
+   i = _config->FindI("Synaptic::ftpProxyPort", 3128);
+   gtk_spin_button_set_value(
+      GTK_SPIN_BUTTON(glade_xml_get_widget(_gladeXML, "spinbutton_ftp_port")), i);
+   str = _config->Find("Synaptic::noProxy", "");
+   gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(_gladeXML,
+                                                     "entry_no_proxy")),
+                      str.c_str());
+
+}
+
+void RGPreferencesWindow::readExpert()
+{
+   // distro selection, block actions here because the optionmenu changes
+   // and a signal is emited
+   _blockAction = true;
+
+   distroChanged = false;
+   string defaultDistro = _config->Find("Synaptic::DefaultDistro", "");
+   int distroMatch = 0;
+      gtk_option_menu_remove_menu(GTK_OPTION_MENU(_optionmenuDefaultDistro));
+   vector<string> archives = _lister->getPolicyArchives();
+   GtkWidget *menu = gtk_menu_new();
+   GtkWidget *mi = gtk_menu_item_new_with_label(_("ignore"));
+   g_object_set_data(G_OBJECT(mi), "me", this);
+   g_signal_connect(G_OBJECT(mi), "activate",
+                    G_CALLBACK(onArchiveSelection), (void *)"");
+   gtk_widget_show(mi);
+   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+   mi = gtk_separator_menu_item_new();
+   gtk_widget_show(mi);
+   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+   for (unsigned int i = 0; i < archives.size(); i++) {
+      //cout << "archive: " << archives[i] << endl;
+      mi = gtk_menu_item_new_with_label(archives[i].c_str());
+      g_object_set_data(G_OBJECT(mi), "me", this);
+      g_signal_connect(G_OBJECT(mi), "activate",
+                       G_CALLBACK(onArchiveSelection),
+                       g_strdup_printf("%s", archives[i].c_str()));
+      gtk_widget_show(mi);
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+      if (defaultDistro == archives[i]) {
+         //cout << "match for: " << archives[i] << endl;
+         // i+2 because we have a ignore at pos 0 and a seperator at pos 1
+         distroMatch = i + 2;
+      }
+   }
+   gtk_option_menu_set_menu(GTK_OPTION_MENU(_optionmenuDefaultDistro), menu);
+   gtk_option_menu_set_history(GTK_OPTION_MENU(_optionmenuDefaultDistro),
+                               distroMatch);
+   _blockAction = false;
+}
+
+void RGPreferencesWindow::show()
+{
+   readGeneral();
+   readColumnsAndFonts();
+   readColors();
+   readTempFiles();
+   readNetwork();
+   readExpert();
+
+   RGWindow::show();
+}
+
+
+void RGPreferencesWindow::readTreeViewValues()
+{
+
+   // number of entries in columns is (sizeof(columns)/sizeof(char*))-1 
+   int number_of_columns = sizeof(column_names)/sizeof(char*)-1;
+
+   // the position in this vector is the position of the column
+   vector<column_struct> columns(number_of_columns);
+
+   // make sure that we fail gracefully
+   bool corrupt=false;
+
+   // put into right place
+   gchar *name;
+   int pos;
+   column_struct c;
+   for(int i=0;column_names[i] != NULL; i++) {
+      // pos
+      name = g_strdup_printf("Synaptic::%sColumnPos",column_names[i]);
+      pos = _config->FindI(name, i);
+      g_free(name);
+      
+      // visible
+      name = g_strdup_printf("Synaptic::%sColumnVisible",column_names[i]);
+      c.visible = _config->FindB(name, true);
+      c.name = column_names[i];
+      c.visible_name = column_visible_names[i];
+      g_free(name);
+
+      if(pos > number_of_columns || pos < 0 || columns[pos].name != NULL) {
+	 cerr << "invalid column config found, reseting"<<endl;
+	 corrupt=true;
+	 continue;
+      }
+      columns[pos] = c;
+   }
+
+   // if corrupt for some reason, repair
+   if(corrupt) {
+      for(int i=0;column_names[i] != NULL; i++) {
+	 name = g_strdup_printf("Synaptic::%sColumnPos",column_names[i]);
+	 _config->Set(name, i);
+	 c.visible = true;
+	 c.name = column_names[i];
+	 c.visible_name = column_visible_names[i];
+	 columns[i] = c;
+      }
+   }
+
+
+   // put into GtkListStore
+   GtkListStore *store = _listColumns = gtk_list_store_new(3, 
+							   G_TYPE_BOOLEAN, 
+							   G_TYPE_STRING, 
+							   G_TYPE_STRING);
+   GtkTreeIter iter;
+   for(unsigned int i=0;i<columns.size();i++) {
+      gtk_list_store_append (store, &iter);
+      gtk_list_store_set (store, &iter,
+			  TREE_CHECKBOX_COLUMN, &columns[i].visible,
+			  TREE_NAME_COLUMN, columns[i].name,
+			  TREE_VISIBLE_NAME_COLUMN, _(columns[i].visible_name),
+			  -1);
+      
+   }
+
+   // now set the model
+   gtk_tree_view_set_model(GTK_TREE_VIEW(_treeView), 
+			   GTK_TREE_MODEL(_listColumns));
+
+}
+
+void RGPreferencesWindow::cbMoveColumnUp(GtkWidget *self, void *data)
+{
+   //cout << "void RGPReferencesWindow::cbMoveColumnUp()" << endl;
+   RGPreferencesWindow *me = (RGPreferencesWindow*)data;
+
+   GtkTreeIter iter, next;
+   GtkTreePath *first = gtk_tree_path_new_first();
+
+   GtkTreeSelection* selection;
+   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_treeView));
+   if(!gtk_tree_selection_get_selected (selection, NULL,
+					&iter)) {
+      return;
+   }
+   next = iter;
+   GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(me->_listColumns), &iter);
+   gtk_tree_path_prev(path);
+   gtk_tree_model_get_iter(GTK_TREE_MODEL(me->_listColumns), &next, path);
+   
+   if(gtk_tree_path_compare(path, first) == 0)
+      gtk_list_store_move_after(me->_listColumns, &iter, NULL);
+   else
+      gtk_list_store_move_before(me->_listColumns, &iter, &next);
+   
+}
+
+void RGPreferencesWindow::cbMoveColumnDown(GtkWidget *self, void *data)
+{
+   //cout << "void RGPReferencesW2indow::cbMoveColumnDown()" << endl;
+   RGPreferencesWindow *me = (RGPreferencesWindow*)data;
+
+   GtkTreeIter iter, next;
+
+   GtkTreeSelection* selection;
+   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(me->_treeView));
+   if(!gtk_tree_selection_get_selected (selection, NULL,
+					&iter)) {
+      return;
+   }
+   next = iter;
+   if(!gtk_tree_model_iter_next(GTK_TREE_MODEL(me->_listColumns), &next))
+      return;
+   gtk_list_store_move_after(me->_listColumns, &iter, &next);
+}
+
+void RGPreferencesWindow::cbToggleColumn(GtkWidget *self, char*path_string,
+					 void *data)
+{
+   //cout << "void RGPReferencesWindow::cbToggle()" << endl;
+   RGPreferencesWindow *me = (RGPreferencesWindow*)data;
+   GtkTreeIter iter;
+   gboolean res;
+
+   GtkTreeModel *model = (GtkTreeModel*)me->_listColumns;
+   GtkTreePath* path = gtk_tree_path_new_from_string(path_string);
+   gtk_tree_model_get_iter(model, &iter, path);
+   gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
+		      TREE_CHECKBOX_COLUMN, &res, -1);
+   gtk_list_store_set(GTK_LIST_STORE(model), &iter,
+		      TREE_CHECKBOX_COLUMN, !res,
+		      -1);
+
+}
+
+
+
+
+
+
 void RGPreferencesWindow::saveColor(GtkWidget *self, void *data)
 {
    GdkColor color;
@@ -619,19 +843,6 @@ void RGPreferencesWindow::checkbuttonUserTerminalFontToggled(GtkWidget *self,
 }
 
 
-void RGPreferencesWindow::hpanedClickedAction(GtkWidget *self, void *data)
-{
-   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
-   //cout << "hpanedClickedAction(GtkWidget *self, void *data) " << endl;
-   me->_synapticLayout = LAYOUT_HPANED;
-}
-
-void RGPreferencesWindow::vpanedClickedAction(GtkWidget *self, void *data)
-{
-   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
-   //cout << "vpanedClickedAction(GtkWidget *self, void *data) " << endl;
-   me->_synapticLayout = LAYOUT_VPANED;
-}
 
 RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
                                          RPackageLister *lister)
@@ -639,7 +850,7 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
 {
    GtkWidget *button;
 
-   _optionUseRegexp = glade_xml_get_widget(_gladeXML, "check_regexp");
+   _optionShowAllPkgInfoInMain = glade_xml_get_widget(_gladeXML, "check_show_all_pkg_info");
    _optionUseStatusColors =
       glade_xml_get_widget(_gladeXML, "check_use_colors");
 
@@ -662,12 +873,16 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
    _optionmenuDel =
       glade_xml_get_widget(_gladeXML, "optionmenu_delbutton_action");
    assert(_optionmenuDel);
-   int delAction = _config->FindI("Synaptic::delAction", PKG_DELETE);
+
+   _optionmenuDefaultDistro =
+      glade_xml_get_widget(_gladeXML, "optionmenu_default_distro");
+   assert(_optionmenuDefaultDistro);
 
    // hide the "remove with configuration" from rpm users
 #ifdef HAVE_RPM
    GtkWidget *w = glade_xml_get_widget(_gladeXML, "menuitem_purge");
    gtk_widget_hide(w);
+   int delAction = _config->FindI("Synaptic::delAction", PKG_DELETE);
    // purge not available 
    if (delAction == PKG_PURGE)
       delAction = PKG_DELETE;
@@ -675,10 +890,6 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
 					"optionmenu_delbutton_action"));
    gtk_widget_hide(glade_xml_get_widget(_gladeXML, "label_removal"));
 #endif
-   // now set the optionmenu
-   // ugly :( but we need this -2 because RGPkgAction starts with 
-   //         "keep","install"
-   gtk_option_menu_set_history(GTK_OPTION_MENU(_optionmenuDel), delAction - 2);
 
    // set data for the checkbutton
    g_object_set_data(G_OBJECT
@@ -692,9 +903,35 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
    // save the lister
    _lister = lister;
 
-   readColors();
+   // treeview stuff
+   _treeView = glade_xml_get_widget(_gladeXML, "treeview_columns");
+   GtkCellRenderer *renderer;
+   GtkTreeViewColumn *column;
+   renderer = gtk_cell_renderer_toggle_new ();
+   g_object_set(renderer, "activatable", TRUE, NULL);
+   g_signal_connect(renderer, "toggled", 
+ 		    (GCallback) cbToggleColumn, this);
+   column = gtk_tree_view_column_new_with_attributes(_("Visible"),
+                                                      renderer,
+                                                      "active", TREE_CHECKBOX_COLUMN,
+                                                      NULL);
+   gtk_tree_view_append_column (GTK_TREE_VIEW(_treeView), column);
+   renderer = gtk_cell_renderer_text_new ();
+   column = gtk_tree_view_column_new_with_attributes (_("Name"),
+						      renderer,
+						      "text", TREE_VISIBLE_NAME_COLUMN,
+						      NULL);
+   gtk_tree_view_append_column (GTK_TREE_VIEW(_treeView), column);
+
+
    readTreeViewValues();
 
+   // lots of signals :)
+   glade_xml_signal_connect_data(_gladeXML, "on_button_column_up_clicked",
+			     G_CALLBACK(cbMoveColumnUp), this);
+   glade_xml_signal_connect_data(_gladeXML, "on_button_column_down_clicked",
+			     G_CALLBACK(cbMoveColumnDown), this);
+
    glade_xml_signal_connect_data(_gladeXML,
                                  "on_close_clicked",
                                  G_CALLBACK(closeAction), this);
@@ -709,13 +946,6 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
                                  "on_button_clean_cache_clicked",
                                  G_CALLBACK(clearCacheAction), this);
 
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_vpaned_clicked",
-                                 G_CALLBACK(vpanedClickedAction), this);
-   glade_xml_signal_connect_data(_gladeXML,
-                                 "on_button_hpaned_clicked",
-                                 G_CALLBACK(hpanedClickedAction), this);
-
    glade_xml_signal_connect_data(_gladeXML,
                                  "on_radio_use_proxy_toggled",
                                  G_CALLBACK(useProxyToggled), this);
@@ -738,43 +968,6 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
                                  GINT_TO_POINTER(FONT_TERMINAL));
 
 
-   // distro selection
-   string defaultDistro = _config->Find("Synaptic::DefaultDistro", "");
-   int distroMatch = 0;
-   _optionmenuDefaultDistro =
-      glade_xml_get_widget(_gladeXML, "optionmenu_default_distro");
-   assert(_optionmenuDefaultDistro);
-   gtk_option_menu_remove_menu(GTK_OPTION_MENU(_optionmenuDefaultDistro));
-   vector<string> archives = _lister->getPolicyArchives();
-   GtkWidget *menu = gtk_menu_new();
-   GtkWidget *mi = gtk_menu_item_new_with_label(_("ignore"));
-   g_object_set_data(G_OBJECT(mi), "me", this);
-   g_signal_connect(G_OBJECT(mi), "activate",
-                    G_CALLBACK(onArchiveSelection), (void *)"");
-   gtk_widget_show(mi);
-   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-   mi = gtk_separator_menu_item_new();
-   gtk_widget_show(mi);
-   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-   for (unsigned int i = 0; i < archives.size(); i++) {
-      //cout << "archive: " << archives[i] << endl;
-      mi = gtk_menu_item_new_with_label(archives[i].c_str());
-      g_object_set_data(G_OBJECT(mi), "me", this);
-      g_signal_connect(G_OBJECT(mi), "activate",
-                       G_CALLBACK(onArchiveSelection),
-                       g_strdup_printf("%s", archives[i].c_str()));
-      gtk_widget_show(mi);
-      gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-      if (defaultDistro == archives[i]) {
-         //cout << "match for: " << archives[i] << endl;
-         // i+2 because we have a ignore at pos 0 and a seperator at pos 1
-         distroMatch = i + 2;
-      }
-   }
-   gtk_option_menu_set_menu(GTK_OPTION_MENU(_optionmenuDefaultDistro), menu);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(_optionmenuDefaultDistro),
-                               distroMatch);
-
    checkbuttonUserTerminalFontToggled(NULL, this);
    checkbuttonUserFontToggled(NULL, this);
 
